@@ -1,74 +1,41 @@
-(function () { // anonymous namespace
+(function (process) {
+
+process.global.process = process;
+process.global.global = process.global;
+global.GLOBAL = global;
 
 /** deprecation errors ************************************************/
 
-GLOBAL.__module = function () {
-  throw new Error("'__module' has been renamed to 'module'");
-};
+function removed (reason) {
+  return function () {
+    throw new Error(reason)
+  }
+}
 
-GLOBAL.include = function () {
-  throw new Error("include(module) has been removed. Use process.mixin(GLOBAL, require(module)) to get the same effect.");
-};
-
-GLOBAL.puts = function () {
-  throw new Error("puts() has moved. Use require('sys') to bring it back.");
-};
-
-GLOBAL.print = function () {
-  throw new Error("print() has moved. Use require('sys') to bring it back.");
-};
-
-GLOBAL.p = function () {
-  throw new Error("p() has moved. Use require('sys') to bring it back.");
-};
-
-process.debug = function () {
-  throw new Error("process.debug() has moved. Use require('sys') to bring it back.");
-};
-
-process.error = function () {
-  throw new Error("process.error() has moved. Use require('sys') to bring it back.");
-};
+GLOBAL.__module = removed("'__module' has been renamed to 'module'");
+GLOBAL.include = removed("include(module) has been removed. Use process.mixin(GLOBAL, require(module)) to get the same effect.");
+GLOBAL.puts = removed("puts() has moved. Use require('sys') to bring it back.");
+GLOBAL.print = removed("print() has moved. Use require('sys') to bring it back.");
+GLOBAL.p = removed("p() has moved. Use require('sys') to bring it back.");
+process.debug = removed("process.debug() has moved. Use require('sys') to bring it back.");
+process.error = removed("process.error() has moved. Use require('sys') to bring it back.");
 
 GLOBAL.node = {};
 
-node.createProcess = function () {
-  throw new Error("node.createProcess() has been changed to process.createChildProcess() update your code");
-};
-
-node.exec = function () {
-  throw new Error("process.exec() has moved. Use require('sys') to bring it back.");
-};
+node.createProcess = removed("node.createProcess() has been changed to process.createChildProcess() update your code");
+node.exec = removed("process.exec() has moved. Use require('sys') to bring it back.");
+node.inherits = removed("node.inherits() has moved. Use require('sys') to access it.");
 
 node.http = {};
-
-node.http.createServer = function () {
-  throw new Error("node.http.createServer() has moved. Use require('http') to access it.");
-};
-
-node.http.createClient = function () {
-  throw new Error("node.http.createClient() has moved. Use require('http') to access it.");
-};
+node.http.createServer = removed("node.http.createServer() has moved. Use require('http') to access it.");
+node.http.createClient = removed("node.http.createClient() has moved. Use require('http') to access it.");
 
 node.tcp = {};
-
-node.tcp.createServer = function () {
-  throw new Error("node.tcp.createServer() has moved. Use require('tcp') to access it.");
-};
-
-node.tcp.createConnection = function () {
-  throw new Error("node.tcp.createConnection() has moved. Use require('tcp') to access it.");
-};
+node.tcp.createServer = removed("node.tcp.createServer() has moved. Use require('tcp') to access it.");
+node.tcp.createConnection = removed("node.tcp.createConnection() has moved. Use require('tcp') to access it.");
 
 node.dns = {};
-
-node.dns.createConnection = function () {
-  throw new Error("node.dns.createConnection() has moved. Use require('dns') to access it.");
-};
-
-node.inherits = function () {
-  throw new Error("node.inherits() has moved. Use require('sys') to access it.");
-};
+node.dns.createConnection = removed("node.dns.createConnection() has moved. Use require('dns') to access it.");
 
 /**********************************************************************/
 
@@ -119,7 +86,7 @@ process.inherits = function (ctor, superCtor) {
 process.createChildProcess = function (file, args, env) {
   var child = new process.ChildProcess();
   args = args || [];
-  env = env || process.ENV;
+  env = env || process.env;
   var envPairs = [];
   for (var key in env) {
     if (env.hasOwnProperty(key)) {
@@ -229,7 +196,7 @@ var eventsModule = createInternalModule('events', function (exports) {
   };
 
   exports.Promise = function () {
-    exports.EventEmitter.call();
+    exports.EventEmitter.call(this);
     this._blocking = false;
     this.hasFired = false;
     this._values = undefined;
@@ -239,45 +206,39 @@ var eventsModule = createInternalModule('events', function (exports) {
   process.Promise = exports.Promise;
 
   exports.Promise.prototype.timeout = function(timeout) {
-    if (timeout === undefined) {
+    if (!timeout) {
       return this._timeoutDuration;
     }
-
+    
     this._timeoutDuration = timeout;
-    if (this._timer) {
-      clearTimeout(this._timer);
-      this._timer = null;
-    }
-
-    var promiseComplete = false;
-    var onComplete = function() {
-      promiseComplete = true;
-      if (this._timer) {
-        clearTimeout(this._timer);
-        this._timer = null;
-      }
-    };
-
-    this
-      .addCallback(onComplete)
-      .addErrback(onComplete);
+    
+    if (this.hasFired) return;
+    this._clearTimeout();
 
     var self = this;
     this._timer = setTimeout(function() {
       self._timer = null;
-      if (promiseComplete) {
+      if (self.hasFired) {
         return;
       }
 
       self.emitError(new Error('timeout'));
-    }, this._timeoutDuration);
+    }, timeout);
 
     return this;
   };
+  
+  exports.Promise.prototype._clearTimeout = function() {
+    if (!this._timer) return;
+    
+    clearTimeout(this._timer);
+    this._timer = null;
+  }
 
   exports.Promise.prototype.emitSuccess = function() {
     if (this.hasFired) return;
-    this.hasFired = true;
+    this.hasFired = 'success';
+    this._clearTimeout();
 
     this._values = Array.prototype.slice.call(arguments);
     this.emit.apply(this, ['success'].concat(this._values));
@@ -285,7 +246,8 @@ var eventsModule = createInternalModule('events', function (exports) {
 
   exports.Promise.prototype.emitError = function() {
     if (this.hasFired) return;
-    this.hasFired = true;
+    this.hasFired = 'error';
+    this._clearTimeout();
 
     this._values = Array.prototype.slice.call(arguments);
     this.emit.apply(this, ['error'].concat(this._values));
@@ -303,7 +265,7 @@ var eventsModule = createInternalModule('events', function (exports) {
   };
 
   exports.Promise.prototype.addCallback = function (listener) {
-    if (this.hasFired) {
+    if (this.hasFired === 'success') {
       return listener.apply(this, this._values);
     }
 
@@ -311,7 +273,7 @@ var eventsModule = createInternalModule('events', function (exports) {
   };
 
   exports.Promise.prototype.addErrback = function (listener) {
-    if (this.hasFired) {
+    if (this.hasFired === 'error') {
       listener.apply(this, this._values);
     }
 
@@ -531,7 +493,7 @@ GLOBAL.clearInterval = GLOBAL.clearTimeout;
 // Modules
 
 var debugLevel = 0;
-if ("NODE_DEBUG" in process.ENV) debugLevel = 1;
+if ("NODE_DEBUG" in process.env) debugLevel = 1;
 
 function debug (x) {
   if (debugLevel > 0) {
@@ -545,11 +507,12 @@ var posixModule = createInternalModule("posix", function (exports) {
   exports.Stats = process.Stats;
 
   function callback (promise) {
-    return function () {
-      if (arguments[0] instanceof Error) {
+    return function (error) {
+      if (error) {
         promise.emitError.apply(promise, arguments);
       } else {
-        promise.emitSuccess.apply(promise, arguments);
+        promise.emitSuccess.apply(promise,
+                                  Array.prototype.slice.call(arguments, 1));
       }
     };
   }
@@ -781,12 +744,12 @@ var path = pathModule.exports;
 process.paths = [ path.join(process.installPrefix, "lib/node/libraries")
                ];
 
-if (process.ENV["HOME"]) {
-  process.paths.unshift(path.join(process.ENV["HOME"], ".node_libraries"));
+if (process.env["HOME"]) {
+  process.paths.unshift(path.join(process.env["HOME"], ".node_libraries"));
 }
 
-if (process.ENV["NODE_PATH"]) {
-  process.paths = process.ENV["NODE_PATH"].split(":").concat(process.paths);
+if (process.env["NODE_PATH"]) {
+  process.paths = process.env["NODE_PATH"].split(":").concat(process.paths);
 }
 
 
@@ -961,13 +924,13 @@ Module.prototype.loadScript = function (filename, loadPromise) {
     require.async = requireAsync;
     require.main = process.mainModule;
     // create wrapper function
-    var wrapper = "var __wrap__ = function (exports, require, module, __filename) { "
+    var wrapper = "(function (exports, require, module, __filename, __dirname) { "
                 + content
-                + "\n}; __wrap__;";
+                + "\n});";
 
     try {
       var compiledWrapper = process.compile(wrapper, filename);
-      compiledWrapper.apply(self.exports, [self.exports, require, self, filename]);
+      compiledWrapper.apply(self.exports, [self.exports, require, self, filename, path.dirname(filename)]);
     } catch (e) {
       loadPromise.emitError(e);
       return;
@@ -1005,19 +968,19 @@ process.exit = function (code) {
 
 var cwd = process.cwd();
 
-// Make process.ARGV[0] and process.ARGV[1] into full paths.
-if (process.ARGV[0].indexOf('/') > 0) {
-  process.ARGV[0] = path.join(cwd, process.ARGV[0]);
+// Make process.argv[0] and process.argv[1] into full paths.
+if (process.argv[0].indexOf('/') > 0) {
+  process.argv[0] = path.join(cwd, process.argv[0]);
 }
 
-if (process.ARGV[1].charAt(0) != "/" && !(/^http:\/\//).exec(process.ARGV[1])) {
-  process.ARGV[1] = path.join(cwd, process.ARGV[1]);
+if (process.argv[1].charAt(0) != "/" && !(/^http:\/\//).exec(process.argv[1])) {
+  process.argv[1] = path.join(cwd, process.argv[1]);
 }
 
 // Load the main module--the command line argument.
 process.mainModule = createModule(".");
 var loadPromise = new events.Promise();
-process.mainModule.load(process.ARGV[1], loadPromise);
+process.mainModule.load(process.argv[1], loadPromise);
 
 // All our arguments are loaded. We've evaluated all of the scripts. We
 // might even have created TCP servers. Now we enter the main eventloop. If
@@ -1028,4 +991,4 @@ process.loop();
 
 process.emit("exit");
 
-}()); // end anonymous namespace
+})
