@@ -2,5 +2,5 @@ process.mixin(require("./common"));
 
 var r = process.memoryUsage();
 puts(inspect(r));
-assertTrue(r["rss"] > 0);
-assertTrue(r["vsize"] > 0);
+assert.equal(true, r["rss"] > 0);
+assert.equal(true, r["vsize"] > 0);
